@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
+#include "private.h"
 
 static void* luv_newuserdata(lua_State* L, size_t sz) {
   void* handle = malloc(sz);
@@ -35,6 +35,15 @@ static uv_handle_t* luv_check_handle(lua_State* L, int index) {
   if (!(udata = lua_touserdata(L, index))) { goto fail; }
   if (!(handle = *(uv_handle_t**) udata)) { goto fail; }
   if (!handle->data) { goto fail; }
+  // "uv_handle" in the registry is a table structured like so:
+  // {
+  //   [<uv_aync metatable>] = true,
+  //   [<uv_check metatable>] = true,
+  //   ...
+  // }
+  // so to check that the value at the index is a "uv_handle",
+  // we get its metatable and check that we get `true` back
+  // when looking the metatable up in the "uv_handle" table.
   lua_getfield(L, LUA_REGISTRYINDEX, "uv_handle");
   lua_getmetatable(L, index < 0 ? index - 1 : index);
   lua_rawget(L, -2);
@@ -97,7 +106,8 @@ static int luv_close(lua_State* L) {
 static void luv_handle_free(uv_handle_t* handle) {
   luv_handle_t* data = (luv_handle_t*)handle->data;
   if (data) {
-    free(data->extra);
+    if (data->extra_gc)
+      data->extra_gc(data->extra);
     free(data);
   }
   free(handle);
@@ -187,3 +197,14 @@ static int luv_fileno(lua_State* L) {
   lua_pushinteger(L, (LUA_INTEGER)(ptrdiff_t)fd);
   return 1;
 }
+
+#if LUV_UV_VERSION_GEQ(1, 19, 0)
+static int luv_handle_get_type(lua_State* L) {
+  uv_handle_t* handle = luv_check_handle(L, 1);
+  uv_handle_type type = uv_handle_get_type(handle);
+  const char* type_name = uv_handle_type_name(type);
+  lua_pushstring(L, type_name);
+  lua_pushinteger(L, type);
+  return 2;
+}
+#endif
